;; Copyright (C) 2008 Anonymous

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.


;; Compiler: Flat assembler (FASM)

;; String dumping works by intercepting text pushdown calls.
;; Every string is dumped only once when it is added and
;; dump is not triggered by successive scrolling.

;; Text pushdown function itself is easily identified by reference
;; to the ASCII string "CDIBMesWnd::AddOutString" at its start.
;; Redirecting the call is a bit trickier, since 'AddOutString' function is referenced from two places.
;; The right one to be redirected is from the function with
;; "CDIBSnrScreen::MoveCmdMESRUN" string reference at its start.

;; Installation.
;; Currently manual only process, a decent HEX editor/disassembler is required.
;; Tune target offsets below, insert assembled code block at the unused space in the end of .text section
;; and redirect text pushdown call to the start of the block.

;; As a useful addition, removing GetACP() calls and hardcoding codepage 932 makes textbuffer work right
;; for non-japanese system locales, although window caption and message boxes will still display garbage.



;; target executable offset config, may differ between builds

;; presented values are valid for games 4-6
GetModuleHandle = 0x0047A108 ;; executable IAT entry address, NOT function entry point
GetProcAddress	= 0x0047A200 ;; executable IAT entry address, NOT function entry point

HookReturnPoint = 0x00401B00 ;; hooked function entry point (CDIBMesWnd::AddOutString)



use32
org 0 ;; using position independent code anyway

;; Hook entry point

push ebp
lea  ebp, [esp+4]

push ebx
push edi
push esi

push ecx  ;; hooking __thiscall function, save 'this' pointer

call @F
db "KERNEL32.DLL",0
@@:
call dword [GetModuleHandle]
mov ebx, eax

call @F
db "CreateFileA",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push 0
push 0
push 4		;; OPEN_ALWAYS
push 0
push 0
push 0xC0000000 ;; READWRITE
call @F
db "dump.txt",0
@@:
call eax  ;; CreateFileA
mov edi, eax ;; fd

;; do append

call @F
db "SetFilePointer",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push 2	  ;; FILE_END
push 0
push 0
push edi  ;; fd
call eax  ;; SetFilePointer

;; write string

call @F
db "lstrlenA",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push dword [ebp+4] ;; ASCIIZ string, first argument of the hooked function
call eax  ;; lstrlenA
mov  esi, eax ;; string length

call @F
db "WriteFile",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push 0
push esp  ;; &written
push esi  ;; string length
push dword [ebp+4] ;; ASCIIZ string, first argument of the hooked function
push edi  ;; fd
call eax  ;; WriteFile

;; and two newlines

call @F
db "WriteFile",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push 0
push esp  ;; &written
push 4
call @F
db 13,10,13,10
@@:
push edi  ;; fd
call eax  ;; WriteFile

;; close

call @F
db "CloseHandle",0
@@:
push ebx  ;; kernel32
call dword [GetProcAddress]
push edi  ;; fd
call eax  ;; CloseHandle

pop ecx   ;; hooking __thiscall function, restore 'this' pointer

pop esi
pop edi
pop ebx
pop ebp

push HookReturnPoint ;; absolute jump to the hooked function
ret